import enum


class MsOrderType(enum.Enum):
    """
    Specifies scan power in scans.
    """
    Ng = -3
    Nl = -2
    Par = -1
    Any = 0
    Ms = 1
    Ms2 = 2
    Ms3 = 3
    Ms4 = 4
    Ms5 = 5
    Ms6 = 6
    Ms7 = 7
    Ms8 = 8
    Ms9 = 9
    Ms10 = 10
